module.exports['menu_seperator.png.http'] = [
  {type: 'file', name: 'image', filename: 'menu_separator.png', fixture: 'menu_separator.png'}
];

module.exports['beta-sticker-1.png.http'] = [
  {type: 'file', name: 'sticker', filename: 'beta-sticker-1.png', fixture: 'beta-sticker-1.png'}
];

module.exports['blank.gif.http'] = [
  {type: 'file', name: 'file', filename: 'blank.gif', fixture: 'blank.gif'}
];

module.exports['binaryfile.tar.gz.http'] = [
  {type: 'file', name: 'file', filename: 'binaryfile.tar.gz', fixture: 'binaryfile.tar.gz'}
];

module.exports['plain.txt.http'] = [
  {type: 'file', name: 'file', filename: 'plain.txt', fixture: 'plain.txt'}
];
