module.exports['generic.http'] = [
  {type: 'file', name: 'upload', filename: '', fixture: 'plain.txt'},
];

module.exports['filename-name.http'] = [
  {type: 'file', name: 'upload', filename: 'plain.txt', fixture: 'plain.txt'},
];
