module.exports['missing-hyphens1.http'] = [
  {type: 'file', name: 'upload', filename: 'plain.txt', fixture: 'plain.txt'},
];
module.exports['missing-hyphens2.http'] = [
  {type: 'file', name: 'upload', filename: 'plain.txt', fixture: 'plain.txt'},
];
